from .base_weight_regularizer import BaseWeightRegularizer
import torch

# modified from http://kaizhao.net/regularface
class RegularFaceRegularizer(BaseWeightRegularizer):
  
  def compute_loss(self, weights):
    num_classes = weights.size(0)
    cos = torch.mm(weights, weights.t())
    if not self.normalize_weights:
        norms = torch.norm(weights, p=2, dim=1, keepdim=True)
        cos = cos / (norms*norms.t())

    cos1 = cos.clone()
    with torch.no_grad():
        row_nums = torch.arange(num_classes).long().to(weights.device)
        cos1[row_nums, row_nums] = -float('inf')
        _, indices = torch.max(cos1, dim=1)
        mask = torch.zeros((num_classes, num_classes)).to(weights.device)
        mask[row_nums, indices] = 1
    
    return torch.sum(cos*mask) / num_classes


from .base_metric_loss_function import BaseMetricLossFunction
from .angular_loss import AngularLoss
from .arcface_loss import ArcFaceLoss
from .contrastive_loss import ContrastiveLoss
from .fast_ap_loss import FastAPLoss
from .generic_pair_loss import GenericPairLoss
from .lifted_structure_loss import GeneralizedLiftedStructureLoss
from .margin_loss import MarginLoss
from .multi_similarity_loss import MultiSimilarityLoss
from .nca_loss import NCALoss
from .normalized_softmax_loss import NormalizedSoftmaxLoss
from .n_pairs_loss import NPairsLoss
from .proxy_losses import ProxyNCALoss
from .signal_to_noise_ratio_losses import SignalToNoiseRatioContrastiveLoss
from .soft_triple_loss import SoftTripleLoss
from .triplet_margin_loss import TripletMarginLoss
from .weight_regularizer_mixin import WeightRegularizerMixin